@@ -3,7 +3,7 @@ package Alien::Base::ModuleBuild;
 use strict;
 use warnings;
 
-our $VERSION = '0.015_01';
+our $VERSION = '0.015_02';
 $VERSION = eval $VERSION;
 
 use parent 'Module::Build';
@@ -403,7 +403,11 @@ sub ACTION_test {
 sub ACTION_install {
   my $self = shift;
   $self->SUPER::ACTION_install;
-  $self->depends_on('alien_install') unless $self->alien_stage_install;
+  if($self->alien_stage_install) {
+    $self->alien_relocation_fixup if $self->alien_stage_install;
+  } else {
+    $self->depends_on('alien_install') unless $self->alien_stage_install;
+  }
 }
 
 sub ACTION_alien_install {
@@ -478,7 +482,8 @@ sub ACTION_alien_install {
     $self->SUPER::ACTION_install;
 
     # refresh the packlist
-    $self->alien_refresh_packlist( $self->alien_library_destination );
+    $self->alien_refresh_packlist( $self->alien_library_destination )
+      unless $self->alien_stage_install;
   }
 }
 
@@ -670,6 +675,10 @@ sub _env_do_system {
     local $CWD;
     pop @CWD;
     
+    my $ldflags = $Config{ldflags};
+    $ldflags .= " -Wl,-headerpad_max_install_names"
+      if $^O eq 'darwin';
+    
     open my $fh, '>', 'config.site';
     print $fh "CC='$Config{cc}'\n";
     # -D define flags should be stripped because they are Perl
@@ -677,7 +686,7 @@ sub _env_do_system {
     print $fh "CFLAGS='", _filter_defines($Config{ccflags}), "'\n";
     print $fh "CPPFLAGS='", _filter_defines($Config{cppflags}), "'\n";
     print $fh "CXXFLAGS='", _filter_defines($Config{ccflags}), "'\n";
-    print $fh "LDFLAGS='$Config{ldflags}'\n";
+    print $fh "LDFLAGS='$ldflags'\n";
     close $fh;
   
     my $config ||= _shell_config_generate();
@@ -1004,6 +1013,37 @@ sub alien_refresh_packlist {
   $packlist->write if $changed;
 }
 
+sub alien_relocation_fixup {
+  my($self) = @_;
+  
+  # so far relocation fixup is only needed on OS X
+  return unless $^O eq 'darwin';
+
+  my $dist_name = $self->dist_name;  
+  my $share = _catdir( $self->install_destination('lib'), qw/auto share dist/, $dist_name );
+  
+  require File::Find;
+  File::Find::find(sub {
+    if(/\.dylib$/)
+    {
+      # save the original mode and make it writable
+      my $mode = (stat $File::Find::name)[2];
+      chmod 0755, $File::Find::name unless -w $File::Find::name;
+      
+      my @cmd = (
+        'install_name_tool',
+        '-id' => $File::Find::name,
+        $File::Find::name,
+      );
+      print "+ @cmd\n";
+      system @cmd;
+      
+      # restore the original permission mode
+      chmod $mode, $File::Find::name;
+    }
+  }, $share);
+}
+
 sub _rscan_destdir {
   my($self, $dir, $pattern) = @_;
   my $destdir = $self->destdir;
